use std::fmt;

use serde::Serialize;

#[derive(Debug, Clone, Serialize)]
pub enum ErrorType {
    DbError,
    Axum,
    Tokio,
}

#[derive(Debug, Clone)]
pub struct AppError {
    pub message: Option<String>,
    pub cause: Option<String>,
    pub error_type: ErrorType,
}

impl AppError {
    pub fn message(&self) -> String {
        match self {
            AppError {
                message: Some(message),
                ..
            } => message.clone(),
            AppError {
                error_type: ErrorType::Tokio,
                ..
            } => "Server Error".to_string(),
            AppError {
                error_type: ErrorType::DbError,
                ..
            } => "The requested item was not found".to_string(),
            _ => "An unexpected error has occurred".to_string(),
        }
    }
}
impl fmt::Display for AppError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
        write!(f, "{}", self.message())
    }
}
// impl From<std::io::Error> for AppError {
//     fn from(error: std::io::Error) -> Self {
//         AppError {
//             message: None,
//             cause: Some(error.to_string()),
//             error_type: ErrorType::Standard,
//         }
//     }
// }

impl From<sqlx::Error> for AppError {
    fn from(error: sqlx::Error) -> AppError {
        AppError {
            message: None,
            cause: Some(error.to_string()),
            error_type: ErrorType::DbError,
        }
    }
}

impl From<axum::Error> for AppError {
    fn from(error: axum::Error) -> Self {
        AppError {
            message: None,
            cause: Some(error.to_string()),
            error_type: ErrorType::Axum,
        }
    }
}

impl From<tokio::io::Error> for AppError {
    fn from(error: tokio::io::Error) -> Self {
        AppError {
            message: None,
            cause: Some(error.to_string()),
            error_type: ErrorType::Tokio,
        }
    }
}
